//      RCParser.cxx
//
//      Copyright 2011, 2014 Andreas Tscharner <andy@vis.ethz.ch>
//
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU Lesser General Public License as
//      published by the Free Software Foundation; either version 3 of the
//      License, or (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU Lesser General Public
//      License along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.


#include <regex>
#include "../StatusCodes.hxx"
#include "../Utils.hxx"
#include "RCParser.hxx"


static inline std::string &left_trim(std::string &s)
{
	s.erase(s.begin(), std::find_if(s.begin(), s.end(), std::not1(std::ptr_fun<int, int>(std::isspace))));
	return s;
}


bool RCParser::is_comment(char *p_line)
{
	int linePos = 0;
	char evalChar;


	while (evalChar = p_line[linePos++]) {
		switch (evalChar) {
			case ' '  :
			case '\t' :
				continue;
			case '#'  :
				return true;
			default   :
				return !( ((unsigned char)evalChar > 32) &&
				          ((unsigned char)evalChar < 127)
				        );
		};
	};
	return true;
}

bool RCParser::is_windows_line(char *p_line)
{
	std::regex winMandatory(R"rgx(^\s*([\w\-]+)\s+(RCDATA)\s+"([\w\-\/\. ]+)")rgx");


	return std::regex_search(p_line, winMandatory);
}

void RCParser::copy_mandatory_data(std::cmatch p_mandatoryMatch, bool p_winData, char **p_resIdent, char **p_resFilename)
{
	size_t idLen;
	size_t filenameLen;
	std::string filenameStr;


	DEBUG_PRINT(("Start RCParser::copy_mandatory_data\n"))
	idLen = p_mandatoryMatch[1].length();
	*p_resIdent = new char[idLen+1];
	memset(*p_resIdent, 0, idLen+1);
	strncpy(*p_resIdent, p_mandatoryMatch[1].str().c_str(), idLen);
	DEBUG_PRINT(("  RCParser::copy_mandatory_data: resIdent: %s\n", *p_resIdent))

	if (p_winData) {
		filenameStr = p_mandatoryMatch[3].str();
	} else {
		filenameStr = p_mandatoryMatch[2].str();
	};
	DEBUG_PRINT(("  RCParser::copy_mandatory_data: filenameStr:  %s\n", filenameStr.c_str()))
	filenameLen = filenameStr.length();
	*p_resFilename = new char[filenameLen+1];
	memset(*p_resFilename, 0, filenameLen+1);
	strncpy(*p_resFilename, filenameStr.c_str(), filenameLen);
	DEBUG_PRINT(("  RCParser::copy_mandatory_data: resFilename: \'%s\'\n", *p_resFilename))
	DEBUG_PRINT(("End RCParser::copy_mandatory_data\n"))
}


RCParser::RCParser(char *p_filename) throw (lrcFileNotFoundException)
	: InFileParser(p_filename)
{
}

int RCParser::parse(void)
{
	const int MAX_LINE_LEN = 512;

	FILE *rcFile;
	ResourceData *entry;
	char oneLine[MAX_LINE_LEN];
	int lineCount = 0;
	char *resIdent;
	char *resFile;
	unsigned char *passwd = nullptr;
	lrc::EncryptionType resEnc;
	lrc::CompressionType resComp;
	std::regex lrcID(R"rgx(^\s*([\w\-]+))rgx");
	std::regex lrcMandatory(R"rgx(^\s*([\w\-]+)\s+"([\w\-\/\. ]+)")rgx");
	std::regex winMandatory(R"rgx(^\s*([\w\-]+)\s+(RCDATA)\s+"([\w\-\/\. ]+)")rgx");
	std::regex lrcCompression(R"rgx(\s*(None|zLib|bzip2))rgx");
	std::regex lrcEncryption(R"rgx(\s*(None|Serpent)(\s+"(.*)")?)rgx");
	std::cmatch mandatoryTokens;
	std::smatch compressionToken;
	std::smatch encryptionTokens;
	std::string tmpRegex;
	bool isWindowsRC = false;
	bool suffixIsEmpty;


	rcFile = fopen(m_filename, "r");
	if (!rcFile) {
		m_lastError = ERROR_FILE_OPEN;
		return ERROR_FILE_OPEN;
	};

	while (fgets(oneLine, MAX_LINE_LEN, rcFile)) {
		lineCount++;
		DEBUG_PRINT(("Parsing file (%s), line: %d...", m_filename, lineCount))

		if (this->is_comment(oneLine)) {
			DEBUG_PRINT(("comment. Go to next line\n"))
			continue;
		};

		if (! std::regex_search(oneLine, mandatoryTokens, lrcMandatory)) {
			DEBUG_PRINT(("no mandatory tokens, trying Windows format..."))
			if (this->is_windows_line(oneLine)) {
				std::regex_search(oneLine, mandatoryTokens, winMandatory);
				DEBUG_PRINT(("yes..."))
				isWindowsRC = true;
			};

			if (!isWindowsRC) {
				DEBUG_PRINT(("no. Closing all files and return\n"))
				fclose(rcFile);
				this->m_lastError = ERROR_PARSE;
				std::get<0>(this->m_errorPosition) = lineCount;
				if (std::regex_search(oneLine, lrcID)) {
					this->m_internalError = ieFilenameNotFound;
					std::get<1>(this->m_errorPosition) = 2;
				} else {
					this->m_internalError = ieIdentNotFound;
					std::get<1>(this->m_errorPosition) = 1;
				};
				return ERROR_PARSE;
			};
		};
		DEBUG_PRINT(("valid line found. Copy data\n"))
		this->copy_mandatory_data(mandatoryTokens, isWindowsRC, &resIdent, &resFile);

		resComp = lrc::NoneCompression;
		resEnc = lrc::NoneEncryption;

		DEBUG_PRINT(("Remaining string: \'%s\' (Length: %d)\n", mandatoryTokens.suffix().str().c_str(), mandatoryTokens.suffix().str().length()))
		suffixIsEmpty = mandatoryTokens.suffix().str().empty() || mandatoryTokens.suffix().str().length() <= 1;
		if ((!suffixIsEmpty) && (!isWindowsRC)) {
			tmpRegex = mandatoryTokens.suffix().str();
			tmpRegex = left_trim(tmpRegex);
			if (std::regex_search(tmpRegex, compressionToken, lrcCompression)) {
				resComp = this->eval_compression_type(compressionToken[1].str().c_str());
				DEBUG_PRINT(("Compression type %d from compression string %s (remains: %s)\n", resComp, compressionToken[1].str().c_str(), compressionToken.suffix().str().c_str()))

				if (!compressionToken.suffix().str().empty()) {
					tmpRegex = compressionToken.suffix().str();
					tmpRegex = left_trim(tmpRegex);
					if (std::regex_search(tmpRegex, encryptionTokens, lrcEncryption)) {
						resEnc = this->eval_encryption_type(encryptionTokens[1].str().c_str());
						DEBUG_PRINT(("Encryption type %d from encryption string %s (Password: %s)\n", resEnc, encryptionTokens[1].str().c_str(), encryptionTokens[3].str().c_str()))

						if (resEnc != lrc::NoneEncryption) {
							try {
								DEBUG_PRINT(("Getting password..."))
								passwd = this->get_password(encryptionTokens[3].str().c_str());
								DEBUG_PRINT(("done: %s\n", passwd))
							} catch (lrcFileNotFoundException const &fnfEx) {
								fclose(rcFile);

								this->m_lastError = ERROR_PARSE;
								this->m_internalError = iePasswordFileNotFound;
								std::get<0>(this->m_errorPosition) = lineCount;
								std::get<1>(this->m_errorPosition) = 5;
								return ERROR_PARSE;
							};

							if (!passwd) {
								fclose(rcFile);

								this->m_lastError = ERROR_PARSE;
								this->m_internalError = ieMissingPassword;
								std::get<0>(this->m_errorPosition) = lineCount;
								std::get<1>(this->m_errorPosition) = 5;
								return ERROR_PARSE;
							};
						};
					};
				} else {
					fclose(rcFile);

					this->m_lastError = ERROR_PARSE;
					this->m_internalError = ieUnknownEncryption;
					std::get<0>(this->m_errorPosition) = lineCount;
					std::get<1>(this->m_errorPosition) = 4;
					return ERROR_PARSE;
				};
			} else {
				fclose(rcFile);

				this->m_lastError = ERROR_PARSE;
				this->m_internalError = ieUnknownCompression;
				std::get<0>(this->m_errorPosition) = lineCount;
				std::get<1>(this->m_errorPosition) = 3;
				return ERROR_PARSE;
			};
		};

		entry = new ResourceData();
		entry->set_ident(resIdent);
		entry->set_file(resFile);
		entry->set_encryption(resEnc, passwd);
		entry->set_compression(resComp);
		entry->set_rc_position(lineCount, -1);
		m_resEntries->push_back(entry);

	};
	fclose(rcFile);

	this->clear_internal_error();
	return NO_ERROR;
}
