//      RCParser.hxx
//
//      Copyright 2011, 2014 Andreas Tscharner <andy@vis.ethz.ch>
//
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU Lesser General Public License as
//      published by the Free Software Foundation; either version 3 of the
//      License, or (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU Lesser General Public
//      License along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.

/*! \file RCParser.hxx
 *
 * This file contains the class definition for the parser for the simple RC
 * format
 *
 * \author Andreas Tscharner
 * \date 2014-10-26
 */

 #ifndef __RCPARSER_HXX__
 #define __RCPARSER_HXX__


// Includefiles
#include <regex>
#include "InFileParser.hxx"


/*! \class RCParser
 *  \brief Class to parse an .rc file
 *
 * Ths RCParser class is responsible to parse a RC file. The RC file format
 * is quite simple and somewhat similar to the Windows rc files.
 */
class RCParser : public InFileParser
{
	private:
		/*! \brief Checks if the line is a comment line
		 *
		 * This method reads the given text as a text line from left to
		 * right abd checks if there are any characters at all and if there
		 * are, if they start with a '#'. An empty line or a line starting
		 * with '#' is treated as comment line
		 *
		 * \param[in] p_line Line of text
		 *
		 * \retval true Line is comment or empty
		 * \retval false Line contains information
		 */
		bool is_comment(char *p_line);
		/*! \brief Checks if the line would be valid for Windows
		 *
		 * This method checks if the given line is a valid line for a
		 * Windows RC file. It will be called whenever the check for the
		 * mandatory lrc entries fail.
		 *
		 * \param[in] p_line Line of RC file
		 *
		 * \retval true It is a valid line for Windows RC
		 * \retval false It would also fail on Windows
		 */
		bool is_windows_line(char *p_line);

		/*! \brief Copy mandatory data in existing variables
		 *
		 * This method copies the mandatory data from a C++ regex match
		 * into the existing variables. It differs between a Windows and
		 * a normal lrc match
		 *
		 * \param[in] p_mandatoryMatch C++ match containing the data
		 * \param[in] p_winData Flag indicating windows
		 * \param[out] p_resIdent Resource identifier
		 * \param[out] p_resFilename Resource filename
		 */
		void copy_mandatory_data(std::cmatch p_mandatoryMatch, bool p_winData, char ** p_resIdent, char **p_resFilename);

	public:
		/*! \brief Constructor
		 *
		 * This is the only constructor of the class and it expects a
		 * filename as parameter
		 *
		 * \param[in] p_filename Filename of RC file
		 *
		 * \exception lrcFileNotFoundException Exception that is thrown if
		 * the given .rc file could not be found
		 */
		RCParser(char *p_filename) throw(lrcFileNotFoundException);

		/*! \brief Parses the file
		 *
		 * This method parses the .rc file, creates a ResourceData class for
		 * each entry and adds them to the internal structure
		 *
		 * \remarks If the method returns ERROR_PARSE, get_internal_error
		 *          will provide more information
		 *
		 * \retval NO_ERROR File successfully parsed
		 * \retval ERROR_FILE_OPEN An error occurred while trying to open
		 *                         the file
		 * \retval ERROR_PARSE An error occurred while trying to parse the
		 *                     file
		 */
		int parse(void);
};


#endif /* __RCPARSER_HXX__ */
