//      EncryptDecyrpt.hxx
//
//      Copyright 2011, 2013, 2014 Andreas Tscharner <andy@vis.ethz.ch>
//
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU Lesser General Public License as
//      published by the Free Software Foundation; either version 3 of the
//      License, or (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU Lesser General Public
//      License along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.


/*! \file EncryptDecrypt.hxx
 *
 * This file contains the abstract class EncryptDecrypt. It is the base
 * class for all encryption and decryption used in \c lrc.
 *
 * \author Andreas Tscharner
 * \date 2014-08-23
 */


#ifndef __ENCRYPTDECRYPT_HXX__
#define __ENCRYPTDECRYPT_HXX__


#include <stddef.h>


namespace lrc
{
	/*! Possible encryption types for resource
	 *
	 * \todo Add more possibilities to encrypt resource
	 */
	enum EncryptionType {
		 NoneEncryption,               //!< No encryption at all
		 SerpentEncryption,            //!< Serpent algorithm for encryption
		 lastEncryption                //!< Marker for last entry
	};

	/*! German names of all possible encryption types for resources
	 */
	static const char *EncryptionTypeNames[] = {"Keine", "Serpent"};

	/*! \class EncryptDecrypt
	 *  \brief Encryption and Decryption base class
	 *
	 * This class is the base class for all encryption and decryption in
	 * \c lrc and \c liblrc
	 */
	class EncryptDecrypt
	{
		public:
			/*! \brief Abstract method for encryption
			 *
			 * This abstract method has to be implemented by a derived class
			 * for encryption
			 *
			 * \param[in] p_key Key for encryption
			 * \param[in] p_clearData Clear resource data for encryption
			 * \param[in] p_clearSize Size of clear resource data
			 * \param[out] p_encData Encrypted resource data
			 * \param[out] p_encSize Size of encrypted data
			 *
			 * \retval NO_ERROR Resource data successfully encrypted
			 * \retval ERROR_INVALID_PARAMETER The return buffer p_encData
			 *                                 was \c nullptr
			 *
			 * \remarks The caller is responsible to free the used memory
			 */
			virtual int encrypt(const unsigned char *p_key, const unsigned char *p_clearData, size_t p_clearSize, unsigned char **p_encData, size_t &p_encSize) = 0;
			/*! \brief Abstract method for decryption
			 *
			 * This abstract method has to be implemented by a derived class
			 * for decryption
			 *
			 * \param[in] p_key Key for decryption
			 * \param[in] p_encData Encrypted resource data for decryption
			 * \param[in] p_encSize Size of encrypted resource data
			 * \param[out] p_clearData Decrypted/clear resource data
			 * \param[out] p_clearSize Size of clear resource data
			 *
			 * \retval NO_ERROR Resource data successfully decrypted
			 * \retval ERROR_INVALID_PARAMETER The return buffer p_clearData
			 *                                 was \c nullptr
			 *
			 * \remarks The caller is responsible to free the used memory
			 */
			virtual int decrypt(const unsigned char *p_key, const unsigned char *p_encData, size_t p_encSize, unsigned char **p_clearData, size_t &p_clearSize) = 0;
	};
}


#endif /* __ENCRYPTDECRYPT_HXX__ */
