//      ResourceManager.cxx
//
//      Copyright 2011, 2012, 2014 Andreas Tscharner <andy@vis.ethz.ch>
//
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU Lesser General Public License as
//      published by the Free Software Foundation; either version 3 of the
//      License, or (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU Lesser General Public
//      License along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.


#include <cstring>
#include <cstdio>
#include "../Utils.hxx"
#include "../StatusCodes.hxx"
#include "../Factories.hxx"
#include "../include/ResourceManager.hxx"


int lrc::ResourceManager::decrypt_data(const unsigned char *p_encData, size_t p_encSize,
                                       unsigned char **p_clearData, size_t &p_clearSize)
{
	const char WHOLE_FILE[] = "Whole file";

	EncryptDecrypt *decryptor;
	int retVal;


	DEBUG_PRINT(("Decrypting data with decryption type %d.", this->m_encType))
	try {
		decryptor = EncryptionFactory::get_encryption_class(this->m_encType, const_cast<char *>(WHOLE_FILE));
	} catch (lrcEncryptionDisabledException const &encDisEx) {
		return ERROR_ENCRYPTION_NOT_AVAILABLE;
	};
	retVal = decryptor->decrypt(this->m_password, p_encData, p_encSize, p_clearData, p_clearSize);
	delete decryptor;
	if (!success(retVal)) {
		DEBUG_PRINT((" Decrypting failed with error code: %d\n", retVal))
		return ERROR_ENCRYPTION_DECRYPT;
	};
	DEBUG_PRINT(("\n"))

	return NO_ERROR;
}

int lrc::ResourceManager::decompress_data(const unsigned char *p_compData, size_t p_compSize,
                                          unsigned char **p_decompData, size_t &p_decompSize)
{
	CompressDecompress *decompressor;
	int retVal;


	DEBUG_PRINT(("Decompressing data with decompressor type %d. ", this->m_compType))
	decompressor = CompressionFactory::get_compression_class(this->m_compType);
	retVal = decompressor->decompress(p_compData, p_compSize, p_decompData, p_decompSize);
	delete decompressor;
	if (!success(retVal)) {
		DEBUG_PRINT(("Decompression failed with error code: %d\n", retVal))
		return ERROR_COMPRESSION_DECOMPRESS;
	};
	DEBUG_PRINT(("Old size: %d; new size: %d\n", p_compSize, p_decompSize))

	return NO_ERROR;
}

int lrc::ResourceManager::setup_resources(const unsigned char *p_resData, size_t p_resSize)
{
	unsigned char *bufferPos;
	unsigned int numEntries;


	bufferPos = const_cast<unsigned char *>(p_resData);

	DEBUG_PRINT(("Getting number of entries..."))
	memcpy(&numEntries, bufferPos, sizeof(unsigned int));
	bufferPos += sizeof(unsigned int);
	DEBUG_PRINT(("done (%d).\n", numEntries))

	DEBUG_PRINT(("Getting all %d entries   ", numEntries))
	this->m_resEntries = new resEntry[numEntries];
	for (int i=0; i<numEntries; i++) {
		memcpy(&m_resEntries[i], bufferPos, sizeof(resEntry));
		bufferPos += sizeof(resEntry);
		DEBUG_PRINT(("."))
	};
	DEBUG_PRINT(("done.\n"))

	this->m_resDataSize = p_resSize - sizeof(unsigned int) - (numEntries * sizeof(resEntry));
	DEBUG_PRINT(("Getting data for all entries (total: %d bytes)\n", this->m_resDataSize))
	this->m_resData = new unsigned char[this->m_resDataSize];
	memset(this->m_resData, 0, this->m_resDataSize);
	memcpy(this->m_resData, bufferPos, this->m_resDataSize);

	this->m_numResEntries = numEntries;

	return NO_ERROR;
}

int lrc::ResourceManager::load_from_file(void)
{
	FILE *rdfFile;
	size_t rdfFileSize;
	size_t decompSize;
	size_t decryptSize;
	int retVal;
	size_t readItems;
	unsigned char *rdfData;
	unsigned char *decompData;
	unsigned char *decryptData;


	DEBUG_PRINT(("Checking current data and entries\n"))
	if (m_resData) {
		delete[] m_resData;
	};
	if (m_resEntries) {
		delete[] m_resEntries;
	};

	DEBUG_PRINT(("Getting file size of %s: ", m_resourceFile))
	rdfFileSize = file_size(m_resourceFile);
	if (rdfFileSize <= 0) {
		return ERROR_FILE_READ;
	};
	DEBUG_PRINT((": %d bytes\n", rdfFileSize))

	rdfFile = fopen(m_resourceFile, "rb");
	if (!rdfFile) {
		return ERROR_FILE_OPEN;
	};

	rdfData = new unsigned char[rdfFileSize];
	DEBUG_PRINT(("Reading %d bytes from file %s (complete file)\n", rdfFileSize, m_resourceFile))
	readItems = fread(rdfData, sizeof(unsigned char), rdfFileSize, rdfFile);
	fclose(rdfFile);

	if (readItems != rdfFileSize) {
		DEBUG_PRINT(("Error reading file. Read %d; expected: %d\n", readItems, rdfFileSize))
		delete[] rdfData;
		return ERROR_FILE_READ;
	};

	retVal = this->decrypt_data(rdfData, rdfFileSize, &decryptData, decryptSize);
	if (is_error(retVal)) {
		delete[] rdfData;
		return retVal;
	};

	retVal = this->decompress_data(decryptData, decryptSize, &decompData, decompSize);
	if (is_error(retVal)) {
		delete[] decryptData;
		return retVal;
	};

	retVal = this->setup_resources(decompData, decompSize);

	delete[] rdfData;
	delete[] decryptData;
	delete[] decompData;

	return retVal;
}

int lrc::ResourceManager::load_embedded(const unsigned char *p_startAddr, const unsigned char *p_endAddr)
{
	size_t rdfDataSize;
	size_t decompSize;
	size_t decryptSize;
	unsigned char *decompData;
	unsigned char *decryptData;
	int retVal;


	DEBUG_PRINT(("Getting data size of embedded resource: "))
	rdfDataSize = p_endAddr - p_startAddr;
	DEBUG_PRINT((": %d bytes\n", rdfDataSize))

	retVal = this->decrypt_data(p_startAddr, rdfDataSize, &decryptData, decryptSize);
	if (is_error(retVal)) {
		return retVal;
	};

	retVal = this->decompress_data(decryptData, decryptSize, &decompData, decompSize);
	if (is_error(retVal)) {
		delete[] decryptData;
		return retVal;
	};

	retVal = this->setup_resources(decompData, decompSize);

	delete[] decryptData;
	delete[] decompData;

	return retVal;
}

lrc::ResourceManager::ResourceManager(const char *p_resFilename,
                                      const lrc::CompressionType p_compress,
                                      const lrc::EncryptionType p_encrypt,
                                      const unsigned char *p_key) throw (lrcFileNotFoundException)
{
	size_t len;


	if (!file_exists((char *)p_resFilename)) {
		throw(lrcFileNotFoundException((char *)p_resFilename));
	};

	len = strlen(p_resFilename);
	m_resourceFile = new char[len+1];
	memset(m_resourceFile, 0, (len+1));
	strncpy(m_resourceFile, p_resFilename, len);

	m_resEntries = nullptr;
	m_numResEntries = -1;
	m_resData = nullptr;
	m_resDataSize = -1;

	m_compType = p_compress;
	m_encType  = p_encrypt;
	m_password = const_cast<unsigned char *>(p_key);
}

lrc::ResourceManager::ResourceManager(const unsigned char *p_startAddr,
                                      const unsigned char *p_endAddr,
                                      const lrc::CompressionType p_compress,
                                      const lrc::EncryptionType p_encrypt,
                                      const unsigned char *p_key)
{
	this->m_resourceFile = nullptr;

	m_resEntries = nullptr;
	m_numResEntries = -1;
	m_resData = nullptr;
	m_resDataSize = -1;

	m_compType = p_compress;
	m_encType  = p_encrypt;
	m_password = const_cast<unsigned char *>(p_key);

	this->load_embedded(p_startAddr, p_endAddr);
}

lrc::ResourceManager::~ResourceManager(void)
{
	if (this->m_resourceFile) {
		delete[] this->m_resourceFile;
	};
}

char **lrc::ResourceManager::get_resource_ids(int &p_numRes)
{
	int retVal, sLen, i;
	char **retIDs = nullptr;


	DEBUG_PRINT(("Checking number of entries: %d\n", m_numResEntries))
	if (m_numResEntries < 0) {
		retVal = this->load_from_file();
		DEBUG_PRINT(("Reading the file returned %d\n", retVal))
		if (!success(retVal)) {
			return nullptr;
		};
	};

	DEBUG_PRINT(("Number of entries after loading the file: %d\n", m_numResEntries))
	retIDs = new char*[m_numResEntries];

	for(i=0; i<m_numResEntries; i++) {
		sLen = strlen(m_resEntries[i].resID);
		retIDs[i] = new char[sLen + 1];
		memset(retIDs[i], 0, (sLen+1));
		strncpy(retIDs[i], m_resEntries[i].resID, sLen);
	};

	p_numRes = m_numResEntries;
	return retIDs;
}

lrc::Resource *lrc::ResourceManager::get_resource(const char *p_resID, const unsigned char *p_password)
{
	int retVal, i, foundID;
	ResourceData *retEntry = nullptr;
	unsigned char *dataStart;


	DEBUG_PRINT(("Checking number of entries: %d\n", m_numResEntries))
	if (m_numResEntries < 0) {
		retVal = this->load_from_file();
		DEBUG_PRINT(("Reading the file returned %d\n", retVal))
		if (!success(retVal)) {
			return nullptr;
		};
	};

	foundID = -1;
	for (i=0; i<m_numResEntries; i++) {
		if (strcmp(m_resEntries[i].resID, p_resID) == 0) {
			foundID = i;
			break;
		};
	};
	if (foundID < 0) {
		return retEntry;
	};

	retEntry = new ResourceData();

	dataStart = m_resData + m_resEntries[foundID].startOffset;
	retVal = retEntry->get_data_from_memory(dataStart, m_resEntries[foundID], p_password);

	if (!success(retVal)) {
		delete retEntry;
		retEntry = nullptr;
	};

	return retEntry;
}

lrc::Resource *lrc::ResourceManager::get_resource(unsigned int p_resIdx, const unsigned char *p_password)
{
	int retVal;
	ResourceData *retEntry;
	unsigned char *dataStart;


	DEBUG_PRINT(("Checking the number of entries: %d\n", m_numResEntries))
	if (m_numResEntries < 0) {
		retVal = this->load_from_file();
		DEBUG_PRINT(("Resing the file returned %d\n", retVal))
		if (!success(retVal)) {
			return nullptr;
		};
	};

	if ((p_resIdx < 0) || (p_resIdx > (m_numResEntries-1))) {
		return nullptr;
	};

	retEntry = new ResourceData();

	dataStart = m_resData + m_resEntries[p_resIdx].startOffset;
	retVal = retEntry->get_data_from_memory(dataStart, m_resEntries[p_resIdx], p_password);

	if (!success(retVal)) {
		delete retEntry;
		retEntry = nullptr;
	};

	return retEntry;
}
