//      SerpentEncryption.hxx
//
//      Copyright 2011, 2014 Andreas Tscharner <andy@vis.ethz.ch>
//
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU Lesser General Public License as
//      published by the Free Software Foundation; either version 3 of the
//      License, or (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU Lesser General Public
//      License along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.


/*! \file SerpentEncryption.hxx
 *
 * This file contains the declaration of the SerpenEncryption class. This
 * class is used to encrypt and decrypt the data using the \em Serpent
 * algorithm
 *
 * \author Andreas Tscharner
 * \date 2014-08-23
 */


#ifndef __SERPENTENCRYPTION_HXX__
#define __SERPENTENCRYPTION_HXX__


#include "../include/EncryptDecrypt.hxx"


/*! \class SerpentEncryption
 *
 * \brief Class to encrypt/decrypt using the \em Serpent algorithm
 *
 * This class encrypts and decrypts the given data using the \em Serpent
 * algorithm. The algorithm is not implemented here, instead the algorithm
 * from the crypto++ library is used
 */
class SerpentEncryption : public lrc::EncryptDecrypt
{
	protected:
		/*! \brief Create initialization vector
		 *
		 * This method creates a random initialization vector for the
		 * Serpent encryption
		 *
		 * \param[in,out] p_ivArray Array for initialization vector
		 * \param[in] p_ivSize Size of the array
		 *
		 * \retval NO_ERROR Initialization vector successfully created
		 * \retval ERROR_INVALID_PARAMETER The array was not created
		 */
		int create_initialization_vector(unsigned char *p_ivArray, size_t p_ivSize);
	public:
		/*! \brief Serpent encryption
		 *
		 * Method to encrypt the given data using the Serpent algorithm
		 */
		int encrypt(const unsigned char *p_key, const unsigned char *p_clearData, size_t p_clearSize, unsigned char **p_encData, size_t &p_encSize);
		/*! \brief Serpent decryption
		 *
		 * Method to decrypt the given data using the Serpent algorithm
		 */
		int decrypt(const unsigned char *p_key, const unsigned char *p_encData, size_t p_encSize, unsigned char **p_clearData, size_t &p_clearSize);
};


#endif /* __SERPENTENCRYPTION_HXX__ */
